@@ -1,4 +1,4 @@
-# Copyright 2016-2020 The Van Valen Lab at the California Institute of
+# Copyright 2016-2021 The Van Valen Lab at the California Institute of
 # Technology (Caltech), with support from the Paul Allen Family Foundation,
 # Google, & National Institutes of Health (NIH) under Grant U24CA224309-01.
 # All rights reserved.
@@ -312,6 +312,47 @@ class Application(object):
 
         return image
 
+    def _batch_predict(self, tiles, batch_size):
+        """Batch process tiles to generate model predictions.
+
+        The built-in keras.predict function has support for batching, but
+        loads the entire image stack into GPU memory, which is prohibitive
+        for large images. This function uses similar code to the underlying
+        model.predict function without soaking up GPU memory.
+
+        Args:
+            tiles (numpy.array): Tiled data which will be fed to model
+            batch_size (int): Number of images to predict on per batch
+
+        Returns:
+            list: Model outputs
+        """
+
+        # list to hold final output
+        output_tiles = []
+
+        # loop through each batch
+        for i in range(0, tiles.shape[0], batch_size):
+            batch_inputs = tiles[i:i + batch_size, ...]
+
+            batch_outputs = self.model.predict(batch_inputs, batch_size=batch_size)
+
+            # model with only a single output gets temporarily converted to a list
+            if not isinstance(batch_outputs, list):
+                batch_outputs = [batch_outputs]
+
+            # initialize output list with empty arrays to hold all batches
+            if not output_tiles:
+                for batch_out in batch_outputs:
+                    shape = (tiles.shape[0],) + batch_out.shape[1:]
+                    output_tiles.append(np.zeros(shape, dtype=tiles.dtype))
+
+            # save each batch to corresponding index in output list
+            for j, batch_out in enumerate(batch_outputs):
+                output_tiles[j][i:i + batch_size, ...] = batch_out
+
+        return output_tiles
+
     def _run_model(self,
                    image,
                    batch_size=4,
@@ -337,7 +378,7 @@ class Application(object):
 
         # Run images through model
         t = timeit.default_timer()
-        output_tiles = self.model.predict(tiles, batch_size=batch_size)
+        output_tiles = self._batch_predict(tiles=tiles, batch_size=batch_size)
         self.logger.debug('Model inference finished in %s s',
                           timeit.default_timer() - t)
 
